@@ -505,8 +505,11 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
 
 void
 _clutter_backend_glx_blit_sub_buffer (ClutterBackendGLX *backend_glx,
-                                      GLXDrawable drawable,
-                                      int x, int y, int width, int height)
+                                      GLXDrawable        drawable,
+                                      int                x,
+                                      int                y,
+                                      int                width,
+                                      int                height)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend_glx);
 
@@ -809,6 +812,7 @@ clutter_backend_glx_create_stage (ClutterBackend  *backend,
   /* copy backend data into the stage */
   stage_x11 = CLUTTER_STAGE_X11 (stage_window);
   stage_x11->wrapper = wrapper;
+  stage_x11->backend = backend_x11;
 
   translator = CLUTTER_EVENT_TRANSLATOR (stage_x11);
   _clutter_backend_x11_add_event_translator (backend_x11, translator);
